/* BEGIN HEADER ===============================================================

    Project:            Arcnet Driver for Windows NT
    Programname:        SIMPLE
    Modulename:
    Filename:           simple.c

    Description:        Simple Example for Controller Access

    Current Version:    4.4
    Created:            11.10.96

    Author:             K. Falkner
    Company:            SoHard AG

    Changes:

    Version Date     ch. by Ch-Idx Ch.-Description
    ------- -------- ------ ------ --------
    1.0     11.10.96 FAL           Creation
    2.0     20.12.96 MEN    [1]    additional DCB-PArameter
    3.0     11.11.97 FAL           Adaption because changes in farc.h
    4.0     23.07.98 MEN           Adaption to FARC-driver V04.00
    4.1     10.01.00 FAL           Treiberzugriffsname als Parameter
    4.2     19.01.00 FAL
    4.3     19.09.00 MEN           Tastendruck nach Programmende erwartet
    4.4     04.10.04 MEN           Bugfix (szDriverName size)

			(c) SoHard AG 1996-2000 All rights reserved
** END HEADER   =============================================================*/

/* ------------------  COMPILER SWITCHES    ------------------*/

/* ------------------      INCLUDES         ------------------*/
#include "windows.h"
#include "stdio.h"
#include "conio.h"
#include "winioctl.h"
#include "stdlib.h"
#include "farc.h"

/* ------------------    DEFINITIONS        ------------------*/

#define VERSION "V4.4"
/* ------------------     CONSTANTS         ------------------*/

/* ------------------       TYPES           ------------------*/

/* ------------------ MODULE-GLOBAL VARIABLES  ---------------*/

/* ------------------  EXTERNAL VARIABLES   ------------------*/

/* ------------------  EXTERNAL FUNCTIONENS ------------------*/


VOID showUsage(
   VOID
   );


int __cdecl main(
   int argc,
   char **argv
   ) {

   HANDLE      hDriver;
   DWORD       dwErrorCode = 0;
   ULONG       dwInterruptCount;
   DWORD       dwBytesReceived, dwBytesWritten;
   ARCNET_DCB  dcb;
   CHAR        szDriverName[256] = "\\\\.\\" ;


   printf("SIMPLE " VERSION ", SOHARD ARCNET driver sample\n\n");
   if ( argc < 3 )  {
      showUsage();
      return 1;
   }

   if ( (dcb.b_node_id = (unsigned char)atoi( argv[2] )) == 0) { // NodeID from 1st Argument
      showUsage();
      return 1;
   }

   strcat(szDriverName, argv[1] );

   if ((hDriver = CreateFile(
      szDriverName,                    // Name
      GENERIC_READ | GENERIC_WRITE,     // Access
      0,                                // ShareMode
      NULL,                             // Security_Attributes
      OPEN_EXISTING,                    // Create
      FILE_ATTRIBUTE_NORMAL,            // File Attributes
      NULL                              // Template
      )) != INVALID_HANDLE_VALUE ) {

      printf( "Driver opened!\n" );

      // initialize Controller

      // set DCB-Structure
      dcb.z_et_1 = TRUE;
      dcb.z_et_2 = TRUE;
      dcb.z_et_3 = FALSE;
      dcb.z_ckp_1 = FALSE;
      dcb.z_ckp_2 = FALSE;
      dcb.z_ckp_3 = FALSE;
      dcb.z_backplane = FALSE;
      dcb.z_p1mode = FALSE;
      dcb.z_four_naks = FALSE;
      dcb.b_init_behaviour = eARC_INIT_BREAK; //[1]

      if (DeviceIoControl(
         hDriver,           // Handle
         IOCTL_FARC_INIT,   // Controlcode
         &dcb,              // Input-Buffer
         sizeof(dcb),       // Bufferlength
         NULL,//buffer,              // Output-Buffer
         0, //sizeof(buffer),                 // Bufferlength
         &dwBytesReceived,  // Bytes read
         NULL               // Pointer to Overlapped Struktur, here not used
         ) == FALSE) {

         dwErrorCode = GetLastError();
         printf( "Controller-Init fails. Errorcode %lu\n", dwErrorCode );


      }
      else {
         char szVersion[26];
         BYTE broadcast[] = {0 ,0, 0,0,0,0, 0xAA};
         broadcast[0] = dcb.b_node_id;

         printf( "Controller-Init success.\n" );

         // send Broadcast


         if ( WriteFile(
            hDriver,
            broadcast,
            sizeof(broadcast),
            &dwBytesWritten,
            NULL
            ) == FALSE) {

            dwErrorCode = GetLastError();
            printf( "Broadcast-Send fails. Errorcode %lu\n", dwErrorCode );

         }
         else {

            printf( "Broadcast-Send success. Bytes sent: %lu\n", dwBytesWritten );

         }

         if ( DeviceIoControl(
            hDriver,
            IOCTL_FARC_GET_INTERRUPTS,
            NULL,
            0,
            &dwInterruptCount,
            sizeof( ULONG ),
            &dwBytesReceived,
            NULL
            ) == TRUE ) {

            printf( "Interrupt counter: %lu\n", dwInterruptCount );
         }

         if ( DeviceIoControl(
            hDriver,
            IOCTL_FARC_GET_SWVERSION,
            NULL,
            0,
            szVersion,
            sizeof( szVersion ),
            &dwBytesReceived,
            NULL
            ) == TRUE ) {

            printf( "Driver version: %s\n", szVersion );
         }
         else {
           dwErrorCode = GetLastError();
           printf( "Version read fails. Errorcode: %lu\n", dwErrorCode );
         }
           CloseHandle( hDriver );
       } // else

   } else {


       dwErrorCode = GetLastError();

       printf("Can't get a handle to Driver %s. Errorcode: %lu\n",argv[1],dwErrorCode);

   }

   // if program not started in console window
   printf("\n\nPress any key...\n");
   do ; while (!kbhit());
   getch();
   return 0;

}


VOID showUsage(VOID)
{
   printf("Usage: simple DriverAccessName NodeId\n"
          "  e.g. simple FARC 4\n"
          "\nPress any key...\n"
          );
   do ; while (!kbhit());
   getch();
}
